package addbk.JAddressBook;

import futils.Futil;
import futils.ReaderUtil;
import futils.WriterUtil;

import java.beans.XMLEncoder;
import java.io.*;
import java.util.Vector;
import java.util.Collections;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;



public class AddressBookDatabase
        implements Serializable {
    private Vector addressVector = new Vector();
    private int recordNumber = 0;

    public static void main(String[] args) {
        AddressRecord abr = AddressRecord.getEmptyDbRecord();
        System.out.println(abr.toString());
    }

    public AddressBookDatabase() {
        addRecord(new AddressRecord());
    }

    public void csvExport() {

        try {
            BufferedWriter bw = WriterUtil.getBufferedWriter("select a CSV file for export");
            for (int i = 0; i <
                    (getNumRecords()); i++) {
                AddressRecord abr = readRecord(i);
                bw.write(abr.getName() + ",");
                bw.write(abr.getAddress() + ",");
                bw.write(abr.getNotes() + ",");
                bw.write(abr.getDial1() + ",");
                bw.write(abr.getDial2() + ",");
                bw.write(abr.getDial3() + ",");
                bw.newLine();
            }
            bw.close();
        } catch (FileNotFoundException e) {
        } catch (IOException e) {
        }

    }

    public void csvMerge() {

        try {
            BufferedReader br = ReaderUtil.getBufferedReader(
                    "select a CSV file");
            for (String line = br.readLine(); line !=
                    null;
                 line = br.readLine())
                addRecord(AddressRecord.getRecord(line));

            br.close();
        } catch (FileNotFoundException e) {
        } catch (IOException e) {
        }
       sort();
    }
    public void saveAsXml(){
        File f = Futil.getWriteFile("Select xml file for output");
        WriterUtil.writeString(f,toString());
    }
    public String toString() {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        XMLEncoder e = new XMLEncoder(baos);
        e.writeObject(this);
        e.flush();
        return baos.toString();
    }

    public void sort() {
        Collections.sort(addressVector);
    }

    public AddressRecord readRecord() {
        return (AddressRecord) addressVector.elementAt(recordNumber);
    }

    public AddressRecord readRecord(int recNumber) {
        return (AddressRecord) addressVector.elementAt(recNumber);
    }


    public void addRecord(AddressRecord abr) {
        System.out.println("added:"+abr);
        addressVector.addElement(abr);
        recordNumber = addressVector.size() - 1;
    }

    public void deleteRecord(AddressRecord abr) {
        addressVector.removeElement(abr);
    }


    public void saveCurrentRecord(AddressRecord abr) {
        try {
            addressVector.setElementAt(abr, recordNumber);
        } catch (ArrayIndexOutOfBoundsException e) {

        }
    }

    public int getNumRecords() {
        return (addressVector.size());
    }

    public void deleteCurrentRecord() {
        addressVector.removeElementAt(recordNumber);
    }

    public void incrementRecord() {
        if (recordNumber < (addressVector.size() - 1))
            recordNumber++;
        System.out.println(recordNumber);
    }

    public void decrementRecord() {
        if (recordNumber > 0)
            recordNumber--;
        System.out.println(recordNumber);
    }

    public void setRecordNumber(int index) {
        if ((index >= 0) && (index <= addressVector.size()))
            recordNumber = index;
        System.out.println(recordNumber);
    }

    public void openGzippedDb() {
        try {
            System.out.println("Opening Address Db Object");
            readDbObject(); 	// gzip format
            //readFromFile();	// object w/o gzip
        } catch (IOException e) {
            System.out.println("IOException");
        } catch (ClassNotFoundException e) {
            System.out.println("ClassNotFoundException");
        }
    }

    public void saveGzippedDb() {
        try {
            System.out.println("Saving Address Db Object");
            saveDbObject();		// gzip format
            //writeToFile();	// object w/o gzip
        } catch (IOException e) {
            System.out.println("IOERROR: " + e.getMessage());
        }
    }

    public void writeToFile() throws IOException {
        FileOutputStream outStream = new FileOutputStream("junkobj");
        System.out.println("FileOutputStream OK");
        ObjectOutputStream ooStream = new ObjectOutputStream(outStream);
        System.out.println("ObjectOutputStream OK");
        ooStream.writeObject(this);
        ooStream.flush();
        ooStream.close();
        outStream.close();
    } // writeToFile()

    public void readFromFile() throws IOException, ClassNotFoundException {
        FileInputStream inStream = new FileInputStream("junkobj");
        ObjectInputStream oiStream = new ObjectInputStream(inStream);
        AddressBookDatabase abd = (AddressBookDatabase) oiStream.readObject();
        this.addressVector = abd.addressVector;
    } // readFromFile()

    public void readDbObject() throws IOException
            , ClassNotFoundException {
        System.out.println("starting read...");
        FileInputStream fis
                = new FileInputStream(
                        Futil.getReadFileName("Select input file"));
        GZIPInputStream gis
                = new GZIPInputStream(fis);
        ObjectInputStream ois
                = new ObjectInputStream(gis);
        AddressBookDatabase abd = (AddressBookDatabase) ois.readObject();
        this.addressVector = abd.addressVector;
        ois.close();
        gis.close();
        fis.close();
        System.out.println(
                "restored v.size: " + this.addressVector.size());
    }// readObject


    public void saveDbObject() throws IOException {
        System.out.println("starting save...");
        FileOutputStream fos
                = new FileOutputStream(
                        Futil.getWriteFile("Select output file"));
        GZIPOutputStream gos
                = new GZIPOutputStream(fos);
        ObjectOutputStream oos
                = new ObjectOutputStream(gos);
        oos.writeObject(this);
        System.out.println("working with save...");
        oos.flush();
        oos.close();
        gos.finish();
        fos.close();
        System.out.println("done with save...");
        System.out.println("saved v.size: " + this.addressVector.size());
    }

    public Vector getAddressVector() {
        return addressVector;
    }

    public void setAddressVector(Vector addressVector) {
        this.addressVector = addressVector;
    }


    public AddressRecord getCurrentRecord() {

        return (AddressRecord)
                addressVector.elementAt(recordNumber);
    }
}


