package addbk.JAddressBook;

import gui.run.RunMenu;
import gui.run.RunMenuBar;
import gui.run.RunMenuItem;
import gui.run.RunButton;

import javax.swing.*;
import java.awt.*;


public final class AddressBookMediator {
    private static AddressBookMediator abm = new AddressBookMediator();

    private AddressRecord storageRecord = new AddressRecord();

    private AddressBookJFrame abf = null;
    private AddressBookDatabase abdb = null;


    private AddressBookMediator() {
        abf = new AddressBookJFrame(getDialPhoneNavigationPanel());
        abf.setJMenuBar(getRunMenuBar());
        abf.setVisible(true);
        abdb = new AddressBookDatabase();
    }
    public DialPhoneNavigationJPanel getDialPhoneNavigationPanel(){
        return new DialPhoneNavigationJPanel(this);
    }
    public static AddressBookMediator getAddressBookMediator() {
        return abm;
    }

    public void nextRecord() {
        abdb.incrementRecord();
        AddressRecord addressRecord = abdb.readRecord();
        setStorageRecord(addressRecord);
        abf.setRecord(addressRecord);
    }

    public void prevRecord() {
        abdb.decrementRecord();
        AddressRecord addressRecord = getAbdb().readRecord();
        setStorageRecord(addressRecord);
        abf.setRecord(addressRecord);
    }

    public void addRecord() {

        AddressRecord abr = abf.getAddressBookRecord();
        abdb.addRecord(abr);
        setStorageRecord(abr);

        setIndexItems();

    }

    public void deleteCurrentRecord() {
        abdb.deleteCurrentRecord();
        setIndexItems();
        setStorageRecord(abdb.readRecord());
    }

    public void revertRecord() {
        abdb.saveCurrentRecord(getStorageRecord());
        setStorageRecord(abdb.readRecord());
        abf.setRecord(abdb.readRecord());

    }


    public void toggleIndexDisplay() {
        // todo display index here
        System.out.println("to be implemented");
    }

    public void setIndexItems() {
        // this should be in the index todo
    }

    public void setRecord(int i) {
        abdb.setRecordNumber(i);
        AddressRecord abr = abdb.readRecord();
        abf.setRecord(abr);
        setStorageRecord(abdb.readRecord());
    }


    public AddressRecord getStorageRecord() {
        return storageRecord;
    }

    public void setStorageRecord(AddressRecord storageRecord) {
        this.storageRecord =
                storageRecord;
    }

    public AddressBookJFrame getAbf() {
        return abf;
    }

    public void setAbf(AddressBookJFrame abf) {
        this.abf = abf;
    }

    public AddressBookDatabase getAbdb() {
        return abdb;
    }


    public AddressBookJFrame getAddressBookFrame() {
        return abf;
    }


    public RunMenuBar getRunMenuBar() {
        RunMenuBar rmb = new RunMenuBar();
        RunMenu fileMenu =
                new RunMenu("[File");
        fileMenu.add(new RunMenuItem("[open gzipped db") {
            public void run() {
                abdb.openGzippedDb();
                abf.setRecord(abdb.readRecord());
                setRecord(0);
                setIndexItems();
            };
        });
        fileMenu.add(new RunMenuItem("[save gzipped db") {
            public void run() {
                abdb.saveGzippedDb();
            };
        });
        fileMenu.add(new RunMenuItem("[merge csv") {
            public void run() {
                abdb.csvMerge();
                setRecord(0);
                setIndexItems();
            };
        });

        fileMenu.add(new RunMenuItem("[export csv") {
            public void run() {
                abdb.csvExport();
            };
        });
        fileMenu.add(new RunMenuItem("[xml out") {
            public void run() {
                abdb.saveAsXml();
            };
        });
        fileMenu.add(new RunMenuItem("[quit{control q}") {
            public void run() {
                System.exit(0);
            };
        });
        rmb.add(fileMenu);
        return rmb;
    }

    public void edit() {
        System.out.println("implement edit here");
    }

    public void done() {
        System.out.println("implement done with edit here");
        //todo finish this
    }

    public void next() {
        System.out.println("implement next here");
        //todo finish this
    }

    public void index() {
        System.out.println("implement index here");
        //todo finish this
    }

    public void previous() {
        System.out.println("previous");
        abf.setRecord(abdb.getCurrentRecord());
    }
    public JPanel getNavigationJPanel() {
        JPanel jp = new JPanel();
        jp.setLayout(new GridLayout(0, 1));
        jp.add(new RunButton("[Previous") {
            public void run() {
                previous();
            }
        });
        jp.add(new RunButton("[next") {
            public void run() {
                next();
            }
        });
        jp.add(new RunButton("[edit") {
            public void run() {
                edit();
            }
        });
        jp.add(new RunButton("[index") {
            public void run() {
                index();
            }
        });
        return jp;
    }
}
